<?xml version="1.0" encoding="ISO-8859-1"?>
<!-- ***** BEGIN LICENSE BLOCK *****
   - Version: MPL 1.1/GPL 2.0/LGPL 2.1
   -
   - The contents of this file are subject to the Mozilla Public License Version
   - 1.1 (the "License"); you may not use this file except in compliance with
   - the License. You may obtain a copy of the License at
   - http://www.mozilla.org/MPL/
   -
   - Software distributed under the License is distributed on an "AS IS" basis,
   - WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
   - for the specific language governing rights and limitations under the
   - License.
   -
   - The Original Code is Forecastfox.
   -
   - The Initial Developer of the Original Code is
   - Jon Stritar <jstritar@MIT.EDU>.
   - Portions created by the Initial Developer are Copyright (C) 2005
   - the Initial Developer. All Rights Reserved.
   -
   - Contributor(s):
   - Jon Stritar <jstritar@MIT.EDU>
   - Richard Klein <richwklein@mchsi.com>
   -
   - Alternatively, the contents of this file may be used under the terms of
   - either the GNU General Public License Version 2 or later (the "GPL"), or
   - the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
   - in which case the provisions of the GPL or the LGPL are applicable instead
   - of those above. If you wish to allow use of your version of this file only
   - under the terms of either the GPL or the LGPL, and not to allow others to
   - use your version of this file under the terms of the MPL, indicate your
   - decision by deleting the provisions above and replace them with the notice
   - and other provisions required by the LGPL or the GPL. If you do not delete
   - the provisions above, a recipient may use your version of this file under
   - the terms of any one of the MPL, the GPL or the LGPL.
   -
   - ***** END LICENSE BLOCK ***** -->

<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0">
  
  <!-- output definition -->
  <xsl:output method="xml" indent="yes"/>

  <!-- root template -->
  <xsl:template match="/">
    <xsl:apply-templates/>
  </xsl:template>

  <!-- profiles template -->
  <xsl:template match="profiles">
    <profiles version="0.7">
      <xsl:apply-templates/>
    </profiles>
  </xsl:template>

  <!-- profile template -->
  <xsl:template match="profile">
    <profile>
      <xsl:attribute name="id">
        <xsl:value-of select="@id"/>
      </xsl:attribute>
      
      <!-- Add new prefs -->
      <pref name="general.position" type="Int" value="-1"/>      
      <pref name="cc.slider.enabled" type="Bool" value="true"/>
      <pref name="cc.slider.display" type="Int" value="2"/>
      <pref name="cc.slider.title" type="Char" value="ff.cc.slider.title"/>
      <pref name="cc.slider.label" type="Char" value="ff.cc.slider.label"/>
      <pref name="cc.tooltip.enabled" type="Bool" value="true"/>
      <pref name="dayf.tooltip.enabled" type="Bool" value="true"/>
      <pref name="dayt.panel.label" type="Char" value="ff.dayt.panel.label"/> 
      <pref name="dayt.tooltip.enabled" type="Bool" value="true"/>
      <pref name="dayt.tooltip.display" type="Int" value="2"/>
      <pref name="dayt.tooltip.label" type="Char" value="ff.dayt.tooltip.label"/>  
                    
      <!-- loop through prefs -->
      <xsl:for-each select="pref">
        <xsl:choose>
          <xsl:when test="@name='locid'">
            <xsl:call-template name="copy">
              <xsl:with-param name="prefname">general.locid</xsl:with-param>            
            </xsl:call-template>
          </xsl:when>        
          <xsl:when test="@name='celcius'">
            <pref>
              <xsl:attribute name="name">general.units</xsl:attribute>
              <xsl:attribute name="type">Int</xsl:attribute>              
              <xsl:if test="@value='true'">
                <xsl:attribute name="value">3</xsl:attribute>
              </xsl:if>
              <xsl:if test="not(@value='true')">
                <xsl:attribute name="value">1</xsl:attribute>
              </xsl:if>           
            </pref>        
          </xsl:when>
          <xsl:when test="@name='migrated'">
            <xsl:call-template name="remove"/>
          </xsl:when>
          <xsl:when test="@name='display.bar'">
            <xsl:call-template name="copy">
              <xsl:with-param name="prefname">general.bar</xsl:with-param>            
            </xsl:call-template>
          </xsl:when>
          <xsl:when test="@name='display.nextnode'">
            <xsl:call-template name="remove"/>
          </xsl:when>            
          <xsl:when test="@name='display.insertafter'">
            <xsl:call-template name="remove"/>
          </xsl:when>          
          <xsl:when test="@name='current.display'">
            <xsl:call-template name="copy">
              <xsl:with-param name="prefname">cc.panel.enabled</xsl:with-param>            
            </xsl:call-template>          
          </xsl:when>  
          <xsl:when test="@name='current.mode'">
            <xsl:call-template name="copy">
              <xsl:with-param name="prefname">cc.panel.display</xsl:with-param>            
            </xsl:call-template>          
          </xsl:when>                
          <xsl:when test="@name='current.cache'">
            <xsl:call-template name="copy">
              <xsl:with-param name="prefname">cc.cache</xsl:with-param>           
            </xsl:call-template>          
          </xsl:when>
          <xsl:when test="@name='current.last'">
            <xsl:call-template name="copy">
              <xsl:with-param name="prefname">cc.last</xsl:with-param>            
            </xsl:call-template>          
          </xsl:when>
          <xsl:when test="@name='current.label'">
            <pref>
              <xsl:attribute name="name">cc.panel.label</xsl:attribute>
              <xsl:attribute name="type">Char</xsl:attribute>              
              <xsl:if test="@value='wf.cc.label'">
                <xsl:attribute name="value">ff.cc.panel.label</xsl:attribute>
              </xsl:if>
              <xsl:if test="not(@value='wf.cc.label')">
                <xsl:attribute name="value">
                  <xsl:value-of select="@value"/>
                </xsl:attribute>
              </xsl:if>           
            </pref>
          </xsl:when>
          <xsl:when test="@name='current.tooltip'">
            <pref>
              <xsl:attribute name="name">cc.tooltip.label</xsl:attribute>
              <xsl:attribute name="type">Char</xsl:attribute>              
              <xsl:if test="@value='wf.cc.tooltip'">
                <xsl:attribute name="value">ff.cc.tooltip.label</xsl:attribute>
              </xsl:if>
              <xsl:if test="not(@value='wf.cc.tooltip')">
                <xsl:attribute name="value">
                  <xsl:value-of select="@value"/>
                </xsl:attribute>
              </xsl:if>           
            </pref>
          </xsl:when>
          <xsl:when test="@name='current.tipicon'">
            <pref>
              <xsl:attribute name="name">cc.tooltip.display</xsl:attribute>
              <xsl:attribute name="type">Int</xsl:attribute>              
              <xsl:if test="@value='true'">
                <xsl:attribute name="value">2</xsl:attribute>
              </xsl:if>
              <xsl:if test="not(@value='true')">
                <xsl:attribute name="value">1</xsl:attribute>
              </xsl:if>           
            </pref>        
          </xsl:when>
          <xsl:when test="@name='forecast.label'">
            <pref>
              <xsl:attribute name="name">dayf.panel.label</xsl:attribute>
              <xsl:attribute name="type">Char</xsl:attribute>              
              <xsl:if test="@value='wf.dayf.label'">
                <xsl:attribute name="value">ff.dayf.panel.label</xsl:attribute>
              </xsl:if>
              <xsl:if test="not(@value='wf.dayf.label')">
                <xsl:attribute name="value">
                  <xsl:value-of select="@value"/>
                </xsl:attribute>
              </xsl:if>           
            </pref>
          </xsl:when>
          <xsl:when test="@name='forecast.parts'">
            <xsl:call-template name="copy">
              <xsl:with-param name="prefname">dayf.panel.mode</xsl:with-param>            
            </xsl:call-template>      
            <xsl:call-template name="copy">
              <xsl:with-param name="prefname">dayt.panel.mode</xsl:with-param>           
            </xsl:call-template>                           
          </xsl:when>
          <xsl:when test="@name='forecast.mode'">
            <xsl:call-template name="copy">
              <xsl:with-param name="prefname">dayf.panel.display</xsl:with-param>            
            </xsl:call-template>      
            <xsl:call-template name="copy">
              <xsl:with-param name="prefname">dayt.panel.display</xsl:with-param>            
            </xsl:call-template>                           
          </xsl:when>
          <xsl:when test="@name='forecast.days'">
            <xsl:call-template name="days"/>
          </xsl:when>                  
          <xsl:when test="@name='forecast.tooltip'">
            <pref>
              <xsl:attribute name="name">dayf.tooltip.label</xsl:attribute>
              <xsl:attribute name="type">Char</xsl:attribute>              
              <xsl:if test="@value='wf.dayf.tooltip'">
                <xsl:attribute name="value">ff.dayf.tooltip.label</xsl:attribute>
              </xsl:if>
              <xsl:if test="not(@value='wf.dayf.tooltip')">
                <xsl:attribute name="value">
                  <xsl:value-of select="@value"/>
                </xsl:attribute>
              </xsl:if>           
            </pref>
          </xsl:when>
          <xsl:when test="@name='forecast.tipicon'">
            <pref>
              <xsl:attribute name="name">dayf.tooltip.display</xsl:attribute>
              <xsl:attribute name="type">Int</xsl:attribute>              
              <xsl:if test="@value='true'">
                <xsl:attribute name="value">2</xsl:attribute>
              </xsl:if>
              <xsl:if test="not(@value='true')">
                <xsl:attribute name="value">1</xsl:attribute>
              </xsl:if>           
            </pref>        
          </xsl:when>                                                           
          <xsl:when test="@name='forecast.cache'">
            <xsl:call-template name="copy">
              <xsl:with-param name="prefname">dayf.cache</xsl:with-param>            
            </xsl:call-template>          
          </xsl:when>
          <xsl:when test="@name='forecast.last'">
            <xsl:call-template name="copy">
              <xsl:with-param name="prefname">dayf.last</xsl:with-param>            
            </xsl:call-template>          
          </xsl:when> 
          <xsl:when test="@name='slider.freq'">
            <xsl:call-template name="copy">
              <xsl:with-param name="prefname">cc.slider.freq</xsl:with-param>            
            </xsl:call-template>          
          </xsl:when>
          <xsl:when test="@name='slider.count'">
            <xsl:call-template name="copy">
              <xsl:with-param name="prefname">cc.slider.count</xsl:with-param>            
            </xsl:call-template>          
          </xsl:when>                                                                                                                                       
          <xsl:otherwise>
            <xsl:call-template name="pref"/>
          </xsl:otherwise>
        </xsl:choose>
      </xsl:for-each>             
    </profile>
  </xsl:template>
  
  <!-- do days based pref -->
  <xsl:template name="days">
    <xsl:if test="@value &gt; '1'">
      <pref>
        <xsl:attribute name="name">dayt.panel.enabled</xsl:attribute>
        <xsl:attribute name="type">Bool</xsl:attribute>
        <xsl:attribute name="value">true</xsl:attribute>
      </pref>
      <pref>
        <xsl:attribute name="name">dayf.panel.enabled</xsl:attribute>
        <xsl:attribute name="type">Bool</xsl:attribute>
        <xsl:attribute name="value">true</xsl:attribute>
      </pref>             
      <pref>
        <xsl:attribute name="name">dayf.panel.days</xsl:attribute>
        <xsl:attribute name="type">Int</xsl:attribute>
        <xsl:attribute name="value">
          <xsl:value-of select="@value - 1"/>
        </xsl:attribute>             
      </pref>
    </xsl:if>
    <xsl:if test="@value='1'">
      <pref>
        <xsl:attribute name="name">dayt.panel.enabled</xsl:attribute>
        <xsl:attribute name="type">Bool</xsl:attribute>
        <xsl:attribute name="value">true</xsl:attribute>
      </pref> 
      <pref>
        <xsl:attribute name="name">dayf.panel.enabled</xsl:attribute>
        <xsl:attribute name="type">Bool</xsl:attribute>
        <xsl:attribute name="value">false</xsl:attribute>
      </pref>            
      <pref>
        <xsl:attribute name="name">dayf.panel.days</xsl:attribute>
        <xsl:attribute name="type">Int</xsl:attribute>
        <xsl:attribute name="value">1</xsl:attribute>             
      </pref>
    </xsl:if>
    <xsl:if test="@value &lt; '1'">
      <pref>
        <xsl:attribute name="name">dayt.panel.enabled</xsl:attribute>
        <xsl:attribute name="type">Bool</xsl:attribute>
        <xsl:attribute name="value">false</xsl:attribute>
      </pref> 
      <pref>
        <xsl:attribute name="name">dayf.panel.enabled</xsl:attribute>
        <xsl:attribute name="type">Bool</xsl:attribute>
        <xsl:attribute name="value">false</xsl:attribute>
      </pref>            
      <pref>
        <xsl:attribute name="name">dayf.panel.days</xsl:attribute>
        <xsl:attribute name="type">Int</xsl:attribute>
        <xsl:attribute name="value">1</xsl:attribute>             
      </pref>
    </xsl:if>         
  </xsl:template>
  
  <!-- remove pref -->
  <xsl:template name="remove"/>
  
  <!-- copy pref and change name -->
  <xsl:template name="copy">
    <xsl:param name="prefname"/>
    <pref>
      <xsl:attribute name="name">
        <xsl:value-of select="$prefname"/>
      </xsl:attribute>
      <xsl:attribute name="type">
        <xsl:value-of select="@type"/>
      </xsl:attribute>
      <xsl:choose>
        <xsl:when test="contains($prefname, 'cache')">
          <xsl:attribute name="value"/>
        </xsl:when>
        <xsl:when test="contains($prefname, 'last')">
          <xsl:attribute name="value"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:attribute name="value">
            <xsl:value-of select="@value"/>
          </xsl:attribute>        
        </xsl:otherwise>        
      </xsl:choose>
    </pref>    
  </xsl:template>
  
  <!-- copy pref -->
  <xsl:template name="pref">
    <pref>
      <xsl:attribute name="name">
        <xsl:value-of select="@name"/>
      </xsl:attribute>
      <xsl:attribute name="type">
        <xsl:value-of select="@type"/>
      </xsl:attribute>
      <xsl:attribute name="value">
        <xsl:value-of select="@value"/>
      </xsl:attribute>
    </pref>
  </xsl:template>
</xsl:stylesheet>
