<?xml version="1.0" encoding="ISO-8859-1"?>
<!-- ***** BEGIN LICENSE BLOCK *****
   - Version: MPL 1.1/GPL 2.0/LGPL 2.1
   -
   - The contents of this file are subject to the Mozilla Public License Version
   - 1.1 (the "License"); you may not use this file except in compliance with
   - the License. You may obtain a copy of the License at
   - http://www.mozilla.org/MPL/
   -
   - Software distributed under the License is distributed on an "AS IS" basis,
   - WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
   - for the specific language governing rights and limitations under the
   - License.
   -
   - The Original Code is Forecastfox.
   -
   - The Initial Developer of the Original Code is
   - Jon Stritar <jstritar@MIT.EDU>.
   - Portions created by the Initial Developer are Copyright (C) 2005
   - the Initial Developer. All Rights Reserved.
   -
   - Contributor(s):
   - Jon Stritar <jstritar@MIT.EDU>
   - Richard Klein <richwklein@mchsi.com>
   -
   - Alternatively, the contents of this file may be used under the terms of
   - either the GNU General Public License Version 2 or later (the "GPL"), or
   - the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
   - in which case the provisions of the GPL or the LGPL are applicable instead
   - of those above. If you wish to allow use of your version of this file only
   - under the terms of either the GPL or the LGPL, and not to allow others to
   - use your version of this file under the terms of the MPL, indicate your
   - decision by deleting the provisions above and replace them with the notice
   - and other provisions required by the LGPL or the GPL. If you do not delete
   - the provisions above, a recipient may use your version of this file under
   - the terms of any one of the MPL, the GPL or the LGPL.
   -
   - ***** END LICENSE BLOCK ***** -->

<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0">
  
  <!-- output definition -->
  <xsl:output method="xml" indent="yes"/>

  <!-- root template -->
  <xsl:template match="/">
    <xsl:apply-templates/>
  </xsl:template>

  <!-- profiles template -->
  <xsl:template match="profiles">
    <profiles version="0.7.5">
      <xsl:apply-templates/>
    </profiles>
  </xsl:template>

  <!-- profile template -->
  <xsl:template match="profile">
    <profile>
      <xsl:attribute name="id">
        <xsl:value-of select="@id"/>
      </xsl:attribute>
      
      <!-- Add new prefs -->
      <pref name="radar.cache" type="Char" value=""/>
      <pref name="radar.panel.enabled" type="Bool" value="true"/>
      <pref name="radar.panel.display" type="Int" value="0"/>
      <pref name="radar.panel.label" type="Char" value="chrome://forecastfox/locale/forecastfox.properties"/> 
      <pref name="radar.tooltip.enabled" type="Bool" value="true"/>
      <pref name="radar.tooltip.display" type="Int" value="0"/>
      <pref name="radar.tooltip.label" type="Char" value="chrome://forecastfox/locale/forecastfox.properties"/> 
      <pref name="swa.panel.enabled" type="Bool" value="true"/>
      <pref name="swa.panel.display" type="Int" value="0"/>
      <pref name="swa.panel.label" type="Char" value="chrome://forecastfox/locale/forecastfox.properties"/> 
      <pref name="swa.tooltip.enabled" type="Bool" value="true"/>
      <pref name="swa.tooltip.display" type="Int" value="2"/>
      <pref name="swa.tooltip.label" type="Char" value="chrome://forecastfox/locale/forecastfox.properties"/>             
      <pref name="swa.slider.enabled" type="Bool" value="true"/>
      <pref name="swa.slider.display" type="Int" value="1"/>
      <pref name="swa.slider.title" type="Char" value="chrome://forecastfox/locale/forecastfox.properties"/>
      <pref name="swa.slider.label" type="Char" value="chrome://forecastfox/locale/forecastfox.properties"/>         

      <!-- loop through prefs -->
      <xsl:for-each select="pref">
        <xsl:choose>
          <xsl:when test="@name='general.locid'">
            <pref>
              <xsl:attribute name="name">
                <xsl:value-of select="@name"/>
              </xsl:attribute>
              <xsl:attribute name="type">
                <xsl:value-of select="@type"/>
              </xsl:attribute>
              <xsl:if test="string-length(@value)='5'">
                <xsl:attribute name="value">
                  <xsl:value-of select="@value"/>
                </xsl:attribute>
              </xsl:if>
              <xsl:if test="not(string-length(@value)='5')">
                <xsl:attribute name="value">00000</xsl:attribute>
              </xsl:if>
            </pref>
          </xsl:when>
          <xsl:when test="@name='links.last'">
            <xsl:call-template name="remove"/>
          </xsl:when>          
          <xsl:when test="@name='links.cache'"> 
            <xsl:call-template name="remove"/>
          </xsl:when>           
          <xsl:when test="contains(@name, 'cache')">
            <xsl:call-template name="clear"/>
          </xsl:when>
          <xsl:when test="contains(@name, 'last')">
            <xsl:call-template name="clear"/>
          </xsl:when>          
          <xsl:when test="contains(@name, 'label')">
            <xsl:call-template name="label"/>
          </xsl:when>
          <xsl:when test="contains(@name, 'title')">
            <xsl:call-template name="label"/>
          </xsl:when>                                                                                    
          <xsl:otherwise>
            <xsl:call-template name="pref"/>
          </xsl:otherwise>
        </xsl:choose>
      </xsl:for-each>
    </profile>
  </xsl:template>

  <!-- label prefs -->
  <xsl:template name="label">
    <xsl:choose>
      <xsl:when test="substring(@value,1,3)='ff.'">
        <pref>
          <xsl:attribute name="name">
            <xsl:value-of select="@name"/>
          </xsl:attribute>
          <xsl:attribute name="type">
            <xsl:value-of select="@type"/>
          </xsl:attribute>
          <xsl:attribute name="value">chrome://forecastfox/locale/forecastfox.properties</xsl:attribute>
        </pref>                   
      </xsl:when>
      <xsl:when test="substring(@value,1,3)='wf.'">
        <pref>
          <xsl:attribute name="name">
            <xsl:value-of select="@name"/>
          </xsl:attribute>
          <xsl:attribute name="type">
            <xsl:value-of select="@type"/>
          </xsl:attribute>
          <xsl:attribute name="value">chrome://forecastfox/locale/forecastfox.properties</xsl:attribute>
        </pref>      
      </xsl:when>      
      <xsl:otherwise>
        <xsl:call-template name="pref"/>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>
  
  <!-- remove pref -->
  <xsl:template name="remove"/>
  
  <!-- clear pref value -->
  <xsl:template name="clear">
    <pref>
      <xsl:attribute name="name">
        <xsl:value-of select="@name"/>
      </xsl:attribute>
      <xsl:attribute name="type">
        <xsl:value-of select="@type"/>
      </xsl:attribute>
      <xsl:attribute name="value"/>
    </pref>  
  </xsl:template>
  
  <!-- copy pref -->
  <xsl:template name="pref">
    <pref>
      <xsl:attribute name="name">
        <xsl:value-of select="@name"/>
      </xsl:attribute>
      <xsl:attribute name="type">
        <xsl:value-of select="@type"/>
      </xsl:attribute>
      <xsl:attribute name="value">
        <xsl:value-of select="@value"/>
      </xsl:attribute>
    </pref>
  </xsl:template>
</xsl:stylesheet>
